import logging

from drf_spectacular.utils import OpenApiTypes, extend_schema
from rest_framework import permissions, status
from rest_framework.response import Response
from rest_framework.views import APIView

log = logging.getLogger(__name__)


class HealthCheck(APIView):
    permission_classes = [permissions.AllowAny]
    authentication_classes = []

    @extend_schema(
        operation_id="healthcheck",
        request=OpenApiTypes.OBJECT,
    )
    def head(self, request, format=None) -> Response:
        try:
            return Response(status=status.HTTP_200_OK)
        except Exception:
            log.exception("HealthCheck Exception")
            return Response("Unknown Error", status=status.HTTP_500_INTERNAL_SERVER_ERROR)



import logging
import os

import requests

from . import constants as c

log = logging.getLogger(__name__)


def callhome_version() -> None:
    build_tag = os.getenv(c.ENV_BUILD_TAG, os.getenv(c.ENV_COMMIT_TAG))
    call_home_token = os.getenv(c.ENV_CALLHOME_TOKEN)
    call_home_url = os.getenv("CALLHOME_URL", "https://jfk-enterprise.com/api/callhome/version/")
    if call_home_token is not None:
        resp = requests.post(
            call_home_url,
            timeout=10,
            headers={
                "Authorization": f"Bearer {call_home_token}",
            },
            json={
                "version": build_tag,
            },
        )
        log.debug(f"Resp {resp}")  # noqa: G004
    else:
        log.warning("No Call Home Token")


def callhome_keep_alive() -> None:
    call_home_token = os.getenv(c.ENV_CALLHOME_TOKEN)
    call_home_keep_alive_url = os.getenv(
        "CALLHOME_KEEP_ALIVE_URL",
        "https://jfk-enterprise.com/api/callhome/keep-alive",
    )
    if call_home_token is not None:
        resp = requests.post(call_home_keep_alive_url, timeout=10)
        log.debug(f"Callhome Resp: {resp}")  # noqa: G004



import logging

from django.contrib.auth.models import User
from knox.auth import TokenAuthentication
from rest_framework import permissions, viewsets
from rest_framework.pagination import PageNumberPagination
from rest_framework.response import Response

from . import serializers

log = logging.getLogger(__name__)


class JFKPagination(PageNumberPagination):
    page_size_query_param = "page_size"
    max_page_size = 1000

    def get_paginated_response(self, data):
        return Response(
            {
                "next": self.page.next_page_number() if self.page.has_next() else None,
                "previous": self.page.previous_page_number() if self.page.has_previous() else None,
                "count": self.page.paginator.count,
                "page": self.page.number,
                "results": data,
            },
        )


class JFKViewSet(viewsets.ModelViewSet):
    """API endpoint that allows users to be viewed or edited."""

    pagination_class = JFKPagination
    permission_classes = [permissions.IsAuthenticated]
    authentication_classes = [TokenAuthentication]


class UserViewSet(JFKViewSet):
    """API endpoint that allows users to be viewed or edited."""

    queryset = User.objects.all()
    serializer_class = serializers.UserSerializer

    def get_queryset(self):
        return super().get_queryset().filter(id=self.request.user.id)



from import_export.admin import ImportExportModelAdmin


class JfkAdmin(ImportExportModelAdmin):
    save_as = True
    save_as_continue = True
    list_display = ("id",)
    list_search = ("id",)
    readonly_fields = ("id",)



"""Django settings for jfk_enterprise_toolbox project.

Generated by 'django-admin startproject' using Django 4.2.8.

For more information on this file, see
https://docs.djangoproject.com/en/4.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/4.2/ref/settings/
"""

import os
from datetime import timedelta
from pathlib import Path

# Base Directory
BASE_DIR = Path(__file__).resolve().parent.parent

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True if os.environ.get("DJANGO_DEBUG", "True") == "True" else False

# Application definition
INSTALLED_APPS = [
    "daphne",  # Daphne for ASGI
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    # 3rd party
    "rest_framework",
    "django_celery_results",
    "django_celery_beat",
    "corsheaders",
    "knox",
    "colorfield",
    "drf_spectacular_sidecar",
    "drf_spectacular",
    "massadmin",
    "extra_settings",
    "django_filters",
    "import_export",
    "channels",
    # Apps
    "jfk_django_core.apps.JfkDjangoCoreConfig",
]


ALLOWED_HOSTS = ["*"]

CORS_ORIGIN_ALLOW_ALL = True
CORS_ALLOW_METHODS = [
    "DELETE",
    "GET",
    "OPTIONS",
    "PATCH",
    "POST",
    "PUT",
]

CSRF_TRUSTED_ORIGINS = os.getenv(
    "DJANGO_CSRF_TRUSTED_ORIGINS",
    "http://localhost:8000,http://localhost:3000,http://localhost:5173",
).split(",")

# Application Middleware
MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    # CORS
    "corsheaders.middleware.CorsMiddleware",
    # Django
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    # 3rd party
    "whitenoise.middleware.WhiteNoiseMiddleware",
]

# Application Templates
TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

# Whitenoise
STATICFILES_STORAGE = "whitenoise.storage.CompressedStaticFilesStorage"

# Database
DATABASES = {
    "default": {
        "ENGINE": os.getenv("DJANGO_DB_ENGINE", "django.db.backends.sqlite3"),
        "NAME": os.getenv("DJANGO_DB_NAME", "db.sqlite3"),
        "USER": os.getenv("DJANGO_DB_USER", "admin"),
        "PASSWORD": os.getenv("DJANGO_DB_PASSWORD", "PTzb7wJE5DkBVE2FMpfY"),
        "HOST": os.getenv("DJANGO_DB_HOST", "192.168.40.6"),
        "PORT": os.getenv("DJANGO_DB_PORT", 5432),
    },
}

# Password validation
# https://docs.djangoproject.com/en/4.2/ref/settings/#auth-password-validators
AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]


# Internationalization
# https://docs.djangoproject.com/en/4.2/topics/i18n/
LANGUAGE_CODE = "en-us"
TIME_ZONE = "UTC"
USE_I18N = True
USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/4.2/howto/static-files/
STATIC_URL = "/static/"
STATIC_ROOT = BASE_DIR / "static"

MEDIA_URL = "/media/"
MEDIA_ROOT = BASE_DIR / "media"

# Default primary key field type
# https://docs.djangoproject.com/en/4.2/ref/settings/#default-auto-field
DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

# Celery
CELERY_BROKER_URL = os.getenv("CELERY_BROKER_URL", "redis://:@localhost:6379")
CELERY_BEAT_SCHEDULER = "django_celery_beat.schedulers:DatabaseScheduler"
CELERY_RESULT_BACKEND = os.getenv("CELERY_BROKER_URL", "redis://:@localhost:6379")
CELERY_ACCEPT_CONTENT = ["application/json"]
CELERY_TASK_SERIALIZER = "json"
CELERY_RESULT_SERIALIZER = "json"

# Logging
LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "default": {
            "format": "{asctime} {levelname} {name} {message}",
            "style": "{",
        },
    },
    "filters": {
        "require_debug_true": {
            "()": "django.utils.log.RequireDebugTrue",
        },
    },
    "handlers": {
        "console": {
            "level": "DEBUG",
            "class": "logging.StreamHandler",
            "formatter": "default",
        },
        "djangofile": {
            "level": "DEBUG",
            "formatter": "default",
            "class": "logging.FileHandler",
            "filename": os.path.join(os.path.join(BASE_DIR, "logs"), "django.log"),
        },
    },
    "loggers": {
        "django.request": {  # for production requests logging and exceptions
            "handlers": ["console", "djangofile"],
            "level": "DEBUG",
            "propagate": False,
        },
        "django.server": {  # for runserver requests logging
            "handlers": ["console", "djangofile"],
            "level": "DEBUG",
            "propagate": False,
        },
    },
    "root": {  # handle all other logs
        "handlers": ["console", "djangofile"],
        "level": "DEBUG",
    },
}

# Rest Framework
REST_FRAMEWORK = {
    "DEFAULT_AUTHENTICATION_CLASSES": (
        #'rest_framework.authentication.BasicAuthentication',
        "rest_framework.authentication.SessionAuthentication",
        "knox.auth.TokenAuthentication",
    ),
    "DEFAULT_PERMISSION_CLASSES": (
        #'rest_framework.permissions.AllowAny',
        "rest_framework.permissions.IsAuthenticated",
    ),
    "DEFAULT_PAGINATION_CLASS": "rest_framework.pagination.PageNumberPagination",
    "PAGE_SIZE": 100,
    "DEFAULT_SCHEMA_CLASS": "drf_spectacular.openapi.AutoSchema",
    "DEFAULT_FILTER_BACKENDS": ["django_filters.rest_framework.DjangoFilterBackend"],
}

# Channel Layers
layersHosts = os.getenv("DJANOG_CHANNEL_LAYERS_HOSTS")
layersHosts = layersHosts.split(",") if layersHosts is not None else None

if layersHosts is not None:
    CHANNEL_LAYERS = {
        "default": {
            "BACKEND": "channels_redis.core.RedisChannelLayer",
            "CONFIG": {
                "hosts": layersHosts,
            },
        },
    }
else:
    # ! No Cross Process Communication
    CHANNEL_LAYERS = {
        "default": {
            "BACKEND": "channels.layers.InMemoryChannelLayer",
        },
    }

# Knox
# "cryptography.hazmat.primitives.hashes.SHA512",
REST_KNOX = {
    "SECURE_HASH_ALGORITHM": "hashlib.sha512",
    "AUTH_TOKEN_CHARACTER_LENGTH": 64,
    "TOKEN_TTL": timedelta(days=14),
    "USER_SERIALIZER": "knox.serializers.UserSerializer",
    "TOKEN_LIMIT_PER_USER": None,
    "AUTO_REFRESH": True,
}

# Spectacular
SPECTACULAR_SETTINGS = {
    "TITLE": "JFK Django Core",
    "DESCRIPTION": "JFK Django Core API",
    "VERSION": "1.0.0",
    "SERVE_INCLUDE_SCHEMA": False,
    "SWAGGER_UI_DIST": "SIDECAR",  # shorthand to use the sidecar instead
    "SWAGGER_UI_FAVICON_HREF": "SIDECAR",
    "REDOC_DIST": "SIDECAR",
}



from django.contrib.auth.models import User
from rest_framework import serializers


class UserSerializer(serializers.ModelSerializer):
    permissions = serializers.SerializerMethodField()

    class Meta:
        model = User
        fields = ["id", "username", "first_name", "last_name", "email", "is_superuser", "permissions"]

    def get_permissions(self, obj):
        return obj.get_all_permissions()
